/*
 *  makeRuleSet(n,maxN): list of rules of length bitsPerRuleSet
 *
 *  Create an array of JSON structures that represent a rule set.
 *  Two example outputs are given below: rule set 110 and rule set 254
 *
 *  Warren Sack <wsack@ucsc.edu>
 *  May 2017
 *  
 */

function makeRuleSet(n,maxN) {

  ////
  //// Example rule sets
  ////

  // rule set 110
  var ruleSet110 =
    [
     {"rule": {"if": ["1","1","1"],
	       "then": "0"}},
     {"rule": {"if": ["1","1","0"],
	       "then": "1"}},
     {"rule": {"if": ["1","0","1"],
	       "then": "1"}},
     {"rule": {"if": ["1","0","0"],
	       "then": "0"}},
     {"rule": {"if": ["0","1","1"],
	       "then": "1"}},
     {"rule": {"if": ["0","1","0"],
	       "then": "1"}},
     {"rule": {"if": ["0","0","1"],
	       "then": "1"}},
     {"rule": {"if": ["0","0","0"],
	       "then": "0"}}
     ];

  // rule set 254
  var ruleSet254 = 
    [
     {"rule": {"if": ["1","1","1"],
	       "then": "1"}},
     {"rule": {"if": ["1","1","0"],
	       "then": "1"}},
     {"rule": {"if": ["1","0","1"],
	       "then": "1"}},
     {"rule": {"if": ["1","0","0"],
	       "then": "1"}},
     {"rule": {"if": ["0","1","1"],
	       "then": "1"}},
     {"rule": {"if": ["0","1","0"],
	       "then": "1"}},
     {"rule": {"if": ["0","0","1"],
	       "then": "1"}},
     {"rule": {"if": ["0","0","0"],
	       "then": "0"}}
     ];

  ////
  //// Ancillary functions
  ////

  //
  // leftPadWithZeroes(s,l): string
  //
  function leftPadWithZeroes(s,l) {
    if ( s.length >= l ) { return(s); }
    else { return(leftPadWithZeroes("0" + s,l)); }
  }

  //
  // decimalToBinary(n): binary number as string
  //
  function decimalToBinary(n) {
    var binaryString = '';
    if ( n === 0 ) { return('0'); }
    while ( n > 0 ) {
      binaryString = ( n % 2 ) + binaryString;
      n = n >> 1;
    }
    return(binaryString);
  }

  //
  // binaryToDecimal(binaryString): integer
  //
  function binaryToDecimal (binaryString) {
    if ( binaryString.length === 1 ) { return(parseInt(binaryString)); }
    else { 
      var current = parseInt(binaryString[0]);
      binaryString = binaryString.substr(1,binaryString.length-1);
      current = current * Math.pow(2,binaryString.length);
      return(current + binaryToDecimal(binaryString));
    }
  }

  //// Create the rule set

  var bitsPerRuleSet = Math.log(maxN+1)/Math.log(2);
  var bitsPerAntecedent = Math.log(bitsPerRuleSet)/Math.log(2);
  var ruleSet = [];
  var antecedent = "";
  var ruleSetConsequents = decimalToBinary(n);
  ruleSetConsequents = leftPadWithZeroes(ruleSetConsequents,bitsPerRuleSet);
  ruleSetConsequents = ruleSetConsequents.split("");
  for ( var i = bitsPerRuleSet-1; i >= 0; i-- ) {
    antecedent = decimalToBinary(i);
    antecedent = leftPadWithZeroes(antecedent,bitsPerAntecedent);
    antecedent = antecedent.split("");
    ruleSet.push({"rule": {"if": antecedent, 
	    "then": ruleSetConsequents[bitsPerRuleSet-1-i]}});
  }
  return(ruleSet);
}
